#!/usr/bin/env python3


# creates a minidump of the currently running python instance
# 2021-04-29, ed <irc.rizon.net>, MIT-licensed
# https://ocv.me/dev/?gist/coreself.py

# minimal subset of https://github.com/skelsec/minidump/blob/master/minidump/utils/createminidump.py
# 2018-05-34, Tamas Jos <skelsecprojects@gmail.com>, MIT-licensed


import os
import time
import ctypes

from ctypes.wintypes import HANDLE, BOOL, DWORD, LPWSTR, LPVOID
from ctypes import windll, WinError, get_last_error


# https://msdn.microsoft.com/en-us/library/windows/desktop/ms684320(v=vs.85).aspx
OpenProcess = ctypes.windll.kernel32.OpenProcess
OpenProcess.argtypes = (DWORD, BOOL, DWORD )
OpenProcess.restype = HANDLE

# https://msdn.microsoft.com/en-us/library/windows/desktop/ms680360(v=vs.85).aspx
MiniDumpWriteDump = ctypes.windll.DbgHelp.MiniDumpWriteDump
MiniDumpWriteDump.argtypes = (HANDLE, DWORD, HANDLE, DWORD, DWORD, DWORD, DWORD)
MiniDumpWriteDump.restype = BOOL

# https://msdn.microsoft.com/en-us/library/windows/desktop/aa363858(v=vs.85).aspx
CreateFile = ctypes.windll.kernel32.CreateFileW
CreateFile.argtypes = (LPWSTR, DWORD, DWORD, LPVOID, DWORD, DWORD, HANDLE)
CreateFile.restype = ctypes.wintypes.HANDLE


FILE_CREATE_ALWAYS = 2
FILE_GENERIC_RW = 0xc0000000
FILE_ATTRIBUTE_NORMAL = 0x80
PROCESS_ALL_ACCESS = 0x1f0fff
DUMPTYPE = 2  # MiniDumpWithFullMemory


def winfug(msg):
	raise Exception('{} ({})'.format(msg, WinError(get_last_error())))


def dump_me():
	pid = os.getpid()
	fn = "{:.3f}.dmp".format(time.time())
	
	hproc = OpenProcess(PROCESS_ALL_ACCESS, False, pid)
	if not hproc:
		winfug('could not openprocess')

	hfile = CreateFile(fn, FILE_GENERIC_RW, 0, None, FILE_CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, None)
	if not hfile:
		winfug('could not createfile')

	res = MiniDumpWriteDump(hproc, pid, hfile, DUMPTYPE, 0, 0, 0)
	if not res:
		winfug('could not writedump')


if __name__ == '__main__':
	dump_me()
	print('aight')
